@@ -119,13 +119,27 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
         res = self.connector._get_iscsi_nodes()
         self.assertEqual([], res)
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_execute')
+    def test_get_iscsi_nodes_corrupt(self, exec_mock):
+        iscsiadm_result = ('ip1:port1,-1 tgt1\n'
+                           'ip2:port2,-1 tgt2\n'
+                           '[]:port3,-1\n'
+                           'ip4:port4,-1 tgt4\n')
+        exec_mock.return_value = (iscsiadm_result, '')
+        res = self.connector._get_iscsi_nodes()
+        expected = [('ip1:port1', 'tgt1'), ('ip2:port2', 'tgt2'),
+                    ('ip4:port4', 'tgt4')]
+        self.assertListEqual(expected, res)
+        exec_mock.assert_called_once_with(
+            'iscsiadm', '-m', 'node', run_as_root=True,
+            root_helper=self.connector._root_helper, check_exit_code=False)
+
     @mock.patch.object(iscsi.ISCSIConnector, '_get_ips_iqns_luns')
     @mock.patch('glob.glob')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_nodes')
     def test_get_connection_devices(self, nodes_mock, sessions_mock,
                                     glob_mock, iql_mock):
-        self.connector.use_multipath = True
         iql_mock.return_value = self.connector._get_all_targets(self.CON_PROPS)
 
         # List sessions from other targets and non tcp sessions
@@ -151,7 +165,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                     ('ip2:port2', 'tgt2'): ({'sdb'}, {'sdc'}),
                     ('ip3:port3', 'tgt3'): (set(), set())}
         self.assertDictEqual(expected, res)
-        iql_mock.assert_called_once_with(self.CON_PROPS, discover=False)
+        iql_mock.assert_called_once_with(self.CON_PROPS, discover=False,
+                                         is_disconnect_call=False)
 
     @mock.patch('glob.glob')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_iscsi_sessions_full')
@@ -545,7 +560,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             mock.sentinel.con_props,
             force=mock.sentinel.Force,
             ignore_errors=mock.sentinel.ignore_errors,
-            device_info=mock.sentinel.dev_info)
+            device_info=mock.sentinel.dev_info,
+            is_disconnect_call=True)
 
     @ddt.data(True, False)
     @mock.patch.object(iscsi.ISCSIConnector, '_get_transport')
@@ -677,19 +693,17 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             (('ip2:port2', 'tgt2'), ({'sdb'}, {'sdc'})),
             (('ip3:port3', 'tgt3'), (set(), set()))))
 
-        with mock.patch.object(self.connector,
-                               'use_multipath') as use_mp_mock:
-            self.connector._cleanup_connection(
-                self.CON_PROPS, ips_iqns_luns=mock.sentinel.ips_iqns_luns,
-                force=False, ignore_errors=False,
-                device_info=mock.sentinel.device_info)
+        self.connector._cleanup_connection(
+            self.CON_PROPS, ips_iqns_luns=mock.sentinel.ips_iqns_luns,
+            force=False, ignore_errors=False,
+            device_info=mock.sentinel.device_info)
 
         get_dev_path_mock.called_once_with(self.CON_PROPS,
                                            mock.sentinel.device_info)
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
-                                              mock.sentinel.ips_iqns_luns)
-        remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
-                                            False, mock.ANY,
+                                              mock.sentinel.ips_iqns_luns,
+                                              False)
+        remove_mock.assert_called_once_with({'sda', 'sdb'}, False, mock.ANY,
                                             path_used, was_multipath)
         discon_mock.assert_called_once_with(
             self.CON_PROPS,
@@ -715,17 +729,16 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             (('ip2:port2', 'tgt2'), ({'sdb'}, {'sdc'})),
             (('ip3:port3', 'tgt3'), (set(), set()))))
 
-        with mock.patch.object(self.connector, 'use_multipath',
-                               wraps=True) as use_mp_mock:
-            self.assertRaises(exception.ExceptionChainer,
-                              self.connector._cleanup_connection,
-                              self.CON_PROPS,
-                              ips_iqns_luns=mock.sentinel.ips_iqns_luns,
-                              force=mock.sentinel.force, ignore_errors=False)
+        self.assertRaises(exception.ExceptionChainer,
+                          self.connector._cleanup_connection,
+                          self.CON_PROPS,
+                          ips_iqns_luns=mock.sentinel.ips_iqns_luns,
+                          force=mock.sentinel.force, ignore_errors=False)
 
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
-                                              mock.sentinel.ips_iqns_luns)
-        remove_mock.assert_called_once_with({'sda', 'sdb'}, use_mp_mock,
+                                              mock.sentinel.ips_iqns_luns,
+                                              False)
+        remove_mock.assert_called_once_with({'sda', 'sdb'},
                                             mock.sentinel.force, mock.ANY,
                                             '', False)
         discon_mock.assert_called_once_with(
@@ -961,6 +974,21 @@ Setting up iSCSI targets: unused
         db_portals_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
         discover_mock.assert_not_called()
 
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_all_targets')
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_discoverydb_portals')
+    @mock.patch.object(iscsi.ISCSIConnector, '_discover_iscsi_portals')
+    def test_get_ips_iqns_luns_disconnect_single_path(self, discover_mock,
+                                                      db_portals_mock,
+                                                      get_targets_mock):
+        db_portals_mock.side_effect = exception.TargetPortalsNotFound
+        res = self.connector._get_ips_iqns_luns(self.SINGLE_CON_PROPS,
+                                                discover=False,
+                                                is_disconnect_call=True)
+        db_portals_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
+        discover_mock.assert_not_called()
+        get_targets_mock.assert_called_once_with(self.SINGLE_CON_PROPS)
+        self.assertEqual(get_targets_mock.return_value, res)
+
     @mock.patch.object(iscsi.ISCSIConnector, '_discover_iscsi_portals')
     def test_get_ips_iqns_luns_no_target_iqns_share_iqn(self, discover_mock):
         discover_mock.return_value = [('ip1:port1', 'tgt1', '1'),
@@ -1206,11 +1234,54 @@ Setting up iSCSI targets: unused
         result = list(get_wwn_mock.call_args[0][0])
         result.sort()
         self.assertEqual(['sda', 'sdb', 'sdc', 'sdd'], result)
+        # Check we pass the mpath
+        self.assertIsNone(get_wwn_mock.call_args[0][1])
         add_wwid_mock.assert_called_once_with('wwn')
         self.assertNotEqual(0, add_path_mock.call_count)
         self.assertGreaterEqual(find_dm_mock.call_count, 2)
         self.assertEqual(4, connect_mock.call_count)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
+                       side_effect=[None, 'dm-0'])
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn', return_value='')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_add_wwid')
+    @mock.patch.object(iscsi.ISCSIConnector, '_connect_vol')
+    @mock.patch('os_brick.utils._time_sleep')
+    def test_connect_multipath_volume_no_wwid(self, sleep_mock, connect_mock,
+                                              add_wwid_mock, add_path_mock,
+                                              get_wwn_mock, find_dm_mock):
+        # Even if we don't have the wwn we'll be able to find the multipath
+        def my_connect(rescans, props, data):
+            devs = {'tgt1': 'sda', 'tgt2': 'sdb', 'tgt3': 'sdc', 'tgt4': 'sdd'}
+            data['stopped_threads'] += 1
+            data['num_logins'] += 1
+            dev = devs[props['target_iqn']]
+            data['found_devices'].append(dev)
+            data['just_added_devices'].append(dev)
+
+        connect_mock.side_effect = my_connect
+
+        with mock.patch.object(self.connector,
+                               'use_multipath'):
+            res = self.connector._connect_multipath_volume(self.CON_PROPS)
+
+        expected = {'type': 'block', 'scsi_wwn': '', 'multipath_id': '',
+                    'path': '/dev/dm-0'}
+        self.assertEqual(expected, res)
+
+        self.assertEqual(3, get_wwn_mock.call_count)
+        result = list(get_wwn_mock.call_args[0][0])
+        result.sort()
+        self.assertEqual(['sda', 'sdb', 'sdc', 'sdd'], result)
+        # Initially mpath we pass is None, but on last call is the mpath
+        mpath_values = [c[1][1] for c in get_wwn_mock._mock_mock_calls]
+        self.assertEqual([None, None, 'dm-0'], mpath_values)
+        add_wwid_mock.assert_not_called()
+        add_path_mock.assert_not_called()
+        self.assertGreaterEqual(find_dm_mock.call_count, 2)
+        self.assertEqual(4, connect_mock.call_count)
+
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
                        side_effect=[None, 'dm-0'])
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwn',
